unit Sql;

interface

uses
{$IFDEF FPC}
  LResources,
{$ENDIF}
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DBCtrls, ExtCtrls, Db, Grids, DBGrids, StdCtrls, ToolWin,
  ComCtrls, Buttons, LiteDacVcl,
  CRFunctions, DBAccess, LiteAccess, DemoFrame, LiteDacDemoForm;

type
  TSqlFrame = class(TDemoFrame)
    ToolBar: TPanel;
    meSQL: TMemo;
    LiteSQL: TLiteSQL;
    Label1: TLabel;
    Splitter1: TSplitter;
    meResult: TMemo;
    btExecute: TSpeedButton;
    btExecInThread: TSpeedButton;
    procedure btExecuteClick(Sender: TObject);
    procedure meSQLExit(Sender: TObject);
    procedure LiteSQLAfterExecute(Sender: TObject; Result: Boolean);
    procedure btExecInThreadClick(Sender: TObject);
  private
    { Private declarations }
    FExecThread: TThread;
    procedure TerminateThread;
  public
    destructor Destroy; override;
    // Demo management
    procedure Initialize; override;
    procedure SetDebug(Value: boolean); override;
  end;

  { TExecThread }

  TExecThread = class(TThread)
  protected
    procedure Execute; override;
    procedure Terminate;
  end;

var
  SqlFrame: TSqlFrame;

implementation

//uses ComObj;

{$IFNDEF FPC}
{$IFDEF CLR}
{$R *.nfm}
{$ELSE}
{$R *.dfm}
{$ENDIF}
{$ENDIF}

procedure LogError(EMessage: string);
begin
  SqlFrame.meResult.Lines.Add('An error with the following message has beein raised during query execution:' + #13#10 + EMessage);
end;

{ TExecThread }

procedure TExecThread.Execute;
begin
  if SqlFrame.LiteSQL <> nil then
    try
      SqlFrame.LiteSQL.Execute;
    except
      on e: Exception do begin
        LogError(e.Message);
      end;
    end;
  Terminate;
end;

procedure TExecThread.Terminate;
begin
  inherited;
end;

{ TSqlFrame }

procedure TSqlFrame.TerminateThread;
begin
  if FExecThread <> nil then begin
    FExecThread.Terminate;
    FExecThread.WaitFor;
    FExecThread.Free;
    FExecThread := nil;
  end;
end;

procedure TSqlFrame.btExecuteClick(Sender: TObject);
begin
  meSQLExit(nil);
  meResult.Lines.Clear;
  LiteDacForm.StatusBar.Panels[2].Text := 'Executing...';
  LiteSQL.Execute;
end;

procedure TSqlFrame.meSQLExit(Sender: TObject);
begin
  if Trim(LiteSQL.SQL.Text) <> Trim(meSQL.Lines.Text) then
    LiteSQL.SQL.Assign(meSQL.Lines);
end;

procedure TSqlFrame.LiteSQLAfterExecute(Sender: TObject; Result: Boolean);
var
  s: string;
  i: integer;
begin
  if Result then
    s := 'Success' + '  (' + IntToStr(LiteSQL.RowsAffected) + ' rows processed)'
  else
    s := 'Execution failed';
  LiteDacForm.StatusBar.Panels[2].Text := s;

  for i := 0 to LiteSQL.Params.Count-1 do
    meResult.Lines.Add(LiteSQL.Params[i].Name + ' = ' + LiteSQL.Params[i].AsString);

  meResult.Lines.Add(s);
end;

destructor TSqlFrame.Destroy;
begin
  TerminateThread;

  inherited;
end;

procedure TSqlFrame.btExecInThreadClick(Sender: TObject);
begin
  AssignStrings(meSQL.Lines, LiteSQL.SQL);
  LiteDacForm.StatusBar.Panels[2].Text := 'Executing...';
  meResult.Lines.Clear;
  TerminateThread;
  FExecThread := TExecThread.Create(False)
end;

// Demo management
procedure TSqlFrame.Initialize;
begin
  inherited;

  FExecThread := nil;
  SqlFrame := Self;
  LiteSQL.Connection := Connection as TLiteConnection;
  AssignStrings(LiteSQL.SQL, meSQL.Lines);
end;

procedure TSqlFrame.SetDebug(Value: boolean);
begin
  LiteSQL.Debug := Value;
end;

{$IFDEF FPC}
initialization
  {$i Sql.lrs}
{$ENDIF}
end.
