
//////////////////////////////////////////////////
//  SQLite Data Access Components
//  Copyright  2024 Devart. All right reserved.
//////////////////////////////////////////////////

{$IFNDEF CLR}
{$I LiteDac.inc}
unit LiteDesign;
{$ENDIF}

interface

uses
{$IFDEF MSWINDOWS}
  Windows, Messages,
{$ENDIF}
  SysUtils, Classes,  Dialogs,
{$IFDEF CLR}
  Borland.Vcl.Design.DesignEditors, Borland.Vcl.Design.DesignIntf,
  Borland.Vcl.Design.FldLinks, WinUtils,
{$ELSE}
  {$IFDEF FPC}
    PropEdits, ComponentEditors,
  {$ELSE}
    {$IFDEF VER6P}
      DesignIntf, DesignEditors,
    {$ELSE}
      DsgnIntf,
    {$ENDIF}
    {$IFNDEF BCB}
      {$IFDEF VER5P}
        FldLinks,
      {$ENDIF}
      ColnEdit,
    {$ENDIF}
  {$ENDIF}
{$ENDIF}
  LiteDacVcl, CRFunctions, CRTypes, LiteAccess, LiteScript, LiteLoader, LiteDump, 
  CRDesign, DADesign;

type
{$IFNDEF FPC}
  TLiteConnectionList = class (TDAConnectionList)
  protected
    function GetConnectionType: TCustomDAConnectionClass; override;
  end;

  TLiteDesignNotification = class(TDADesignNotification)
  public
    procedure ItemInserted(const ADesigner: IDesigner; AItem: TPersistent); override;
    function CreateConnectionList: TDAConnectionList; override;
    function GetConnectionPropertyName: string; override;
  end;
{$ENDIF}

{ ------------  LiteDac property editors ----------- }
  TLiteFileNameProperty = class(TStringProperty)
  protected
    procedure GetDialogOptions(Dialog: TOpenDialog); virtual;
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TLiteClientLibraryProperty = class(TStringProperty)
  protected
    procedure GetDialogOptions(Dialog: TOpenDialog); virtual;
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TLiteConnectDialogPropertyEditor = class(TComponentProperty)
  private
    FCheckProc: TGetStrProc;
    procedure CheckComponent(const Value: string);
  public
    procedure GetValues(Proc: TGetStrProc); override;
  end;

{ ------------  LiteDac component editors ----------- }
  TLiteConnectionEditor = class (TDAConnectionEditor)
  protected
    procedure InitVerbs; override;
  end;

  TLiteSQLEditor = class (TDASQLEditor)
  protected
    procedure InitVerbs; override;
  end;

  TLiteQueryEditor = class (TDAComponentEditor)
  protected
    procedure InitVerbs; override;
  end;

  TLiteTableEditor = class (TDAComponentEditor)
  protected
    procedure InitVerbs; override;
  end;

  TLiteScriptEditor = class (TDAScriptEditor)
  protected
    procedure InitVerbs; override;
  end;

  TLiteUpdateSQLEditor = class (TDAUpdateSQLEditor)
  protected
    procedure InitVerbs; override;
  end;

{$IFNDEF STD}
  TLiteUserFunctionEditor = class (TDAComponentEditor)
  protected
    procedure InitVerbs; override;
  end;
{$ENDIF}

{$IFNDEF FPC}
{ ------------  LiteDac selection editors ----------- }
  TLiteConnectionSelectionEditor = class(TSelectionEditor)
  public
    procedure RequiresUnits(Proc: TGetStrProc); override;
  end;
{$ENDIF}

procedure Register;

implementation

uses
  DBAccess,
  DAScriptEditor,
  LiteDesignUtils,
  LiteConnectionEditor, LiteSQLEditor, LiteQueryEditor,
  LiteUpdateSQLEditor, LiteUserFunctionEditor,
{$IFNDEF FPC}
  LiteMenu,
{$ENDIF}  
  LiteTableEditor;

{$IFNDEF FPC}
var
  Notificator: IDesignNotification;
{$ENDIF}

procedure Register;
begin
  // Register property editors
  RegisterPropertyEditor(TypeInfo(String), TLiteConnection, 'Database', TLiteFileNameProperty);
  RegisterPropertyEditor(TypeInfo(String), TLiteConnection, 'EncryptionKey', TDAPasswordProperty);
{$IFNDEF FPC}
  RegisterSelectionEditor(TLiteConnection, TLiteConnectionSelectionEditor);
{$ENDIF}
{$IFNDEF CLR}
  RegisterPropertyEditor(TypeInfo(String), TLiteConnection, 'ClientLibrary', TLiteClientLibraryProperty);
{$ENDIF}
  RegisterPropertyEditor(TypeInfo(TStrings), TLiteSQL, 'SQL', TDAPropertyEditor);

  RegisterPropertyEditor(TypeInfo(TCustomConnectDialog), TLiteConnection, 'ConnectDialog', TLiteConnectDialogPropertyEditor);

  RegisterPropertyEditor(TypeInfo(TStrings), TLiteQuery, 'SQL', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TLiteQuery, 'SQLDelete', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TLiteQuery, 'SQLInsert', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TLiteQuery, 'SQLLock', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TLiteQuery, 'SQLRefresh', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TLiteQuery, 'SQLUpdate', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TLiteQuery, 'SQLRecCount', TDAPropertyEditor);

  RegisterPropertyEditor(TypeInfo(TStrings), TLiteScript, 'SQL', TDAPropertyEditor);

  RegisterPropertyEditor(TypeInfo(TStrings), TLiteUpdateSQL, 'InsertSQL', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TLiteUpdateSQL, 'ModifySQL', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TLiteUpdateSQL, 'DeleteSQL', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TLiteUpdateSQL, 'RefreshSQL', TDAPropertyEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TLiteUpdateSQL, 'LockSQL', TDAPropertyEditor);

// Register component editors
  DARegisterComponentEditor(TLiteConnection, TLiteConnectionEditor, TLiteConnectionEditorForm, TLiteDesignUtils);
  DARegisterComponentEditor(TLiteSQL, TLiteSQLEditor, TLiteSQLEditorForm, TLiteDesignUtils);
  DARegisterComponentEditor(TLiteQuery, TLiteQueryEditor, TLiteQueryEditorForm, TLiteDesignUtils);
  DARegisterComponentEditor(TLiteTable, TLiteTableEditor, TLiteTableEditorForm, TLiteDesignUtils);
  DARegisterComponentEditor(TLiteScript, TLiteScriptEditor, TDAScriptEditorForm, TLiteDesignUtils);
  DARegisterComponentEditor(TLiteUpdateSQL, TLiteUpdateSQLEditor, TLiteUpdateSQLEditorForm, TLiteDesignUtils);
{$IFNDEF FPC}
  DARegisterComponentEditor(TLiteDataSource, TCRDataSourceEditor, nil, TLiteDesignUtils);
{$ENDIF}
{$IFNDEF STD}
  DARegisterComponentEditor(TLiteLoader, TDALoaderEditor, nil, TLiteDesignUtils);
  DARegisterComponentEditor(TLiteDump, TDAComponentEditor, nil, TLiteDesignUtils);
  DARegisterComponentEditor(TLiteUserFunction, TLiteUserFunctionEditor, nil, TLiteDesignUtils);
  DARegisterComponentEditor(TLiteUserCollation, TDAComponentEditor, nil, TLiteDesignUtils);
{$ENDIF}

{$IFNDEF FPC}
  Menu.AddItems({$IFDEF CLR}WinUtils{$ELSE}SysInit{$ENDIF}.HInstance);
{$ENDIF}
end;

{ TLiteFileNameProperty }

procedure TLiteFileNameProperty.GetDialogOptions(Dialog: TOpenDialog);
begin
{$IFDEF MSWINDOWS}
  Dialog.Filter := 'SQLite Database Files (*.db3, *.db, *.sqlite)|*.db3;*.db;*.sqlite|All Files (*.*)|*.*';
{$ELSE}
  Dialog.Filter := 'All Files (*)|*';
{$ENDIF}
  Dialog.Options := Dialog.Options + [ofFileMustExist];
end;

procedure TLiteFileNameProperty.Edit;
var
  OpenDialog: TOpenDialog;
begin
  OpenDialog := TOpenDialog.Create(nil);
  GetDialogOptions(OpenDialog);
  if OpenDialog.Execute then
    SetValue(OpenDialog.FileName);
  OpenDialog.Free;
end;

function TLiteFileNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paRevertable, paDialog];
end;

{ TLiteClientLibraryProperty }

procedure TLiteClientLibraryProperty.GetDialogOptions(Dialog: TOpenDialog);
begin
{$IFDEF MSWINDOWS}
  Dialog.Filter := 'Dynamic link library (*.dll)|*.dll|All Files (*.*)|*.*';
{$ELSE}
  Dialog.Filter := 'All Files (*)|*';
{$ENDIF}
  Dialog.Options := Dialog.Options + [ofFileMustExist];
end;

procedure TLiteClientLibraryProperty.Edit;
var
  OpenDialog: TOpenDialog;
begin
  OpenDialog := TOpenDialog.Create(nil);
  GetDialogOptions(OpenDialog);
  if OpenDialog.Execute then
    SetValue(OpenDialog.FileName);
  OpenDialog.Free;
end;

function TLiteClientLibraryProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paRevertable, paDialog];
end;

{ TLiteConnectDialogPropertyEditor }

procedure TLiteConnectDialogPropertyEditor.CheckComponent(const Value: string);
var
  Component: TComponent;
begin
  Component := {$IFDEF FPC}PropertyHook{$ELSE}Designer{$ENDIF}.GetComponent(Value);
  if Component <> nil then begin
    if not ({$IFDEF VER17P}(GetIsClassByName(Component, 'TLiteConnectDialogFmx')) or {$ENDIF}(Component is TLiteConnectDialog)) then
      Exit;
  end;
  FCheckProc(Value);
end;

procedure TLiteConnectDialogPropertyEditor.GetValues(Proc: TGetStrProc);
begin
  FCheckProc := Proc;
  inherited GetValues(CheckComponent);
end;

{ TLiteConnectionEditor }

procedure TLiteConnectionEditor.InitVerbs;
begin
  inherited;

  AddVerb('Connection Editor...', TLiteConnectionEditorForm, TLiteDesignUtils);
end;

{ TLiteQueryEditor }

procedure TLiteQueryEditor.InitVerbs;
begin
  AddVerb('Fields &Editor...', ShowFieldsEditor);
  AddVerb('LiteQuery E&ditor...', TLiteQueryEditorForm, TLiteDesignUtils);
  AddVerb('Data Editor...', ShowDataEditor);
  
  inherited;
end;

{ TLiteSQLEditor }

procedure TLiteSQLEditor.InitVerbs;
begin
  inherited;

  AddVerb('LiteSQL E&ditor...', TLiteSQLEditorForm, TLiteDesignUtils);
end;

{ TLiteScriptEditor }

procedure TLiteScriptEditor.InitVerbs;
begin
  inherited;

  AddVerb('LiteScript E&ditor...', TDAScriptEditorForm, TLiteDesignUtils);
end;

{ TLiteUpdateSQLEditor }

procedure TLiteUpdateSQLEditor.InitVerbs;
begin
  inherited;

  AddVerb('LiteUpdateSQL E&ditor...', TLiteUpdateSQLEditorForm, TLiteDesignUtils);
end;

{ TLiteTableEditor }

procedure TLiteTableEditor.InitVerbs;
begin
  AddVerb('Fields &Editor...', ShowFieldsEditor);
  AddVerb('LiteTable E&ditor...', TLiteTableEditorForm, TLiteDesignUtils);
  AddVerb('Data Editor...', ShowDataEditor);

  inherited;
end;

{ TLiteUserFunctionEditor }

{$IFNDEF STD}
procedure TLiteUserFunctionEditor.InitVerbs;
begin
  AddVerb('LiteUserFunction E&ditor...', TLiteUserFunctionEditorForm, TLiteDesignUtils);

  inherited;
end;
{$ENDIF}

{$IFNDEF FPC}
{ TLiteConnectionSelectionEditor }

procedure TLiteConnectionSelectionEditor.RequiresUnits(Proc: TGetStrProc);
begin
  Proc('LiteCall');
  Proc('LiteConsts');
end;

{ TLiteDesignNotification }

function TLiteDesignNotification.CreateConnectionList: TDAConnectionList;
begin
  Result := TLiteConnectionList.Create;
end;

function TLiteDesignNotification.GetConnectionPropertyName: string;
begin
  Result := 'Connection';
end;

procedure TLiteDesignNotification.ItemInserted(const ADesigner: IDesigner;
  AItem: TPersistent);
begin
  if (AItem <> nil) and ((AItem is TCustomLiteDataSet)
    or (AItem is TLiteScript)
    or (AItem is TLiteDataSource)
    or (AItem is TLiteSQL)
  {$IFNDEF STD}
    or (AItem is TLiteLoader)
    or (AItem is TLiteDump)
    or (AItem is TLiteMetaData)
    or (AItem is TLiteUserFunction)
    or (AItem is TLiteUserCollation)
  {$ENDIF}
    )
  then
    FItem := AItem;
end;

{ TLiteConnectionList }

function TLiteConnectionList.GetConnectionType: TCustomDAConnectionClass;
begin
  Result := TLiteConnection;
end;
{$ENDIF}

{$IFNDEF FPC}
initialization
  Notificator := TLiteDesignNotification.Create;
  RegisterDesignNotification(Notificator);

finalization
  UnRegisterDesignNotification(Notificator);
  Notificator := nil;
{$ENDIF}

end.
