
//////////////////////////////////////////////////
//  SQLite Data Access Components
//  Copyright  2024 Devart. All right reserved.
//////////////////////////////////////////////////

{$IFNDEF CLR}
{$I LiteDac.inc}

unit LiteDesignUtils;
{$ENDIF}

interface

uses
  Classes, SysUtils, Dialogs,
  CRTypes, CRDataTypeMap, CRDesignUtils,
  DBAccess, DADesignUtils;

type
  TLiteDesignUtils = class(TDADesignUtils)
    class function GetProjectName: string; override;

  { Component }
    class function GetDesignCreate(Obj: TComponent): boolean; override;
    class procedure SetDesignCreate(Obj: TComponent; Value: boolean); override;

  { Connection support }
    class function HasConnection(Obj: TComponent): boolean; override;
    class function GetConnection(Obj: TComponent): TCustomDAConnection; override;
    class procedure SetConnection(Obj: TComponent; Value: TCustomDAConnection); override;

  { SQL support }
    class function GetParams(Obj: TComponent): TDAParams; override;
    class procedure SetParams(Obj: TComponent; Value: TDAParams); override;

  { TDATable support }
    class function GetTableName(Obj: TCustomDADAtaSet): string; override;
    class procedure SetTableName(Obj: TCustomDADAtaSet; const Value: string); override;
    class function GetOrderFields(Obj: TCustomDADAtaSet): string; override;
    class procedure SetOrderFields(Obj: TCustomDADAtaSet; const Value: string); override;
    class procedure PrepareSQL(Obj: TCustomDADAtaSet); override;

  { Data Type Mapping }
    class function GetConverterManagerClass: TConverterManagerClass; override;
  end;

implementation

uses
  LiteAccess, LiteDataTypeMap;

{ TLiteDesignUtils }

class function TLiteDesignUtils.GetConnection(
  Obj: TComponent): TCustomDAConnection;
begin
  Assert(Obj <> nil);
{$IFNDEF STD}
  if (Obj is TCustomLiteUserFunction) then
    Result := TCustomLiteUserFunction(Obj).Connection
  else if (Obj is TCustomLiteUserCollation) then
    Result := TCustomLiteUserCollation(Obj).Connection
  else
{$ENDIF}
    Result := inherited GetConnection(Obj);
end;

class function TLiteDesignUtils.GetConverterManagerClass: TConverterManagerClass;
begin
  Result := TConverterManagerClass(TLiteMapRules.GetConverterManager.ClassType);
end;

class function TLiteDesignUtils.GetDesignCreate(Obj: TComponent): boolean;
begin
  Result := inherited GetDesignCreate(Obj);
end;

class function TLiteDesignUtils.GetOrderFields(Obj: TCustomDADAtaSet): string;
begin
  Assert(Obj is TCustomLiteTable, Obj.ClassName);
  Result := TCustomLiteTable(Obj).OrderFields;
end;

class function TLiteDesignUtils.GetParams(Obj: TComponent): TDAParams;
begin
  Assert(Obj <> nil);
{$IFNDEF STD}
  if Obj is TCustomLiteUserFunction then
    Result := TCustomLiteUserFunction(Obj).Params
  else
{$ENDIF}
    Result := inherited GetParams(Obj);
end;

class function TLiteDesignUtils.GetProjectName: string;
begin
  Result := 'LiteDAC';
end;

class function TLiteDesignUtils.GetTableName(Obj: TCustomDADAtaSet): string;
begin
  Assert(Obj is TCustomLiteTable, Obj.ClassName);
  Result := TCustomLiteTable(Obj).TableName;
end;

class function TLiteDesignUtils.HasConnection(Obj: TComponent): boolean;
begin
  Result := inherited HasConnection(Obj);
end;

class procedure TLiteDesignUtils.PrepareSQL(Obj: TCustomDADAtaSet);
begin
  Assert(Obj is TCustomLiteTable, Obj.ClassName);
  TCustomLiteTable(Obj).PrepareSQL;
end;

class procedure TLiteDesignUtils.SetConnection(Obj: TComponent;
  Value: TCustomDAConnection);
begin
  Assert(Obj <> nil);
{$IFNDEF STD}
  if (Obj is TCustomLiteUserFunction) then
    TCustomLiteUserFunction(Obj).Connection := TLiteConnection(Value)
  else if (Obj is TCustomLiteUserCollation) then
    TCustomLiteUserCollation(Obj).Connection := TLiteConnection(Value)
  else
{$ENDIF}
    inherited;
end;

class procedure TLiteDesignUtils.SetDesignCreate(Obj: TComponent;
  Value: boolean);
begin
  inherited;
end;

class procedure TLiteDesignUtils.SetOrderFields(Obj: TCustomDADAtaSet;
  const Value: string);
begin
  Assert(Obj is TCustomLiteTable, Obj.ClassName);
  TCustomLiteTable(Obj).OrderFields := Value;
end;

class procedure TLiteDesignUtils.SetParams(Obj: TComponent; Value: TDAParams);
begin
  Assert(Obj <> nil);
{$IFNDEF STD}
  if (Obj is TCustomLiteUserFunction) then
    TCustomLiteUserFunction(Obj).Params := TLiteUserFunctionParams(Value)
  else
{$ENDIF}
    inherited;
end;

class procedure TLiteDesignUtils.SetTableName(Obj: TCustomDADAtaSet;
  const Value: string);
begin
  Assert(Obj is TCustomLiteTable, Obj.ClassName);
  TCustomLiteTable(Obj).TableName := Value;
end;

end.
