type

{ TLiteConnectDialog }

{$IFDEF VER16P}
  [ComponentPlatformsAttribute(pidWin32 or pidWin64{$IFNDEF STD} or pidOSX32{$IFDEF VER25P} or pidLinux64{$IFDEF VER26P} or pidOSX64{$ENDIF}{$ENDIF}{$ENDIF})]
{$ENDIF}
  {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF} = class(TCustomConnectDialog)
  private
    FDatabaseOption: TConnectDialogOption;
    FDirectOption: TConnectDialogOption;
    FClientLibraryOption: TConnectDialogOption;

    function GetDatabaseLabel: string;
    procedure SetDatabaseLabel(Value: string);
    function GetShowDatabase: Boolean;
    procedure SetShowDatabase(Value: Boolean);
    function GetShowPassword: Boolean;
    procedure SetShowPassword(Value: Boolean);

    procedure SetDatabaseOption(Value: TConnectDialogOption);
    procedure SetDirectOption(Value: TConnectDialogOption);
    procedure SetClientLibraryOption(Value: TConnectDialogOption);

    function GetConnection: TLiteConnection;

  protected
    function DefDialogClass: TClass; override;
    procedure SetLabelSet(Value: TLabelSet); override;

  {$IFDEF MSWINDOWS}
    function GetKeyPath: string; override;
    procedure SaveInfoToRegistry(Registry: TRegistry); override;
    procedure LoadInfoFromRegistry(Registry: TRegistry); override;
    procedure SaveServerListToRegistry; override;
    function GetServerStoreName: string; override;
  {$ENDIF}

    procedure ReadDatabaseCaptionProperty(Reader: TReader);
    procedure ReadShowDatabaseProperty(Reader: TReader);
    procedure ReadShowPasswordProperty(Reader: TReader);
    procedure DefineProperties(Filer: TFiler); override;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;

    function Execute: boolean; override;
  {$IFDEF MSWINDOWS}
    procedure LoadServerListFromRegistry(List: TStrings); reintroduce;
  {$ENDIF}
    function BrowseForDatabase: string;
    function BrowseForClientLibrary: string;

    procedure GetOptions(var Options: TConnectDialogOptionArray; Ordered: Boolean = True); override;

    property Connection: TLiteConnection read GetConnection;

    property ShowDatabase: boolean read GetShowDatabase write SetShowDatabase default True;
    property ShowPassword: boolean read GetShowPassword write SetShowPassword default True;
    property DatabaseLabel: string read GetDatabaseLabel write SetDatabaseLabel;
  published
    property Retries;
    property SavePassword;
    property DialogClass;

    property Caption;
    property ConnectButton;
    property CancelButton;

    property Password;
    property Database: TConnectDialogOption read FDatabaseOption write SetDatabaseOption;
    property Direct: TConnectDialogOption read FDirectOption write SetDirectOption;
    property ClientLibrary: TConnectDialogOption read FClientLibraryOption write SetClientLibraryOption;

    property LabelSet;
    property StoreLogInfo;
  end;

function DefConnectDialogClass: TClass;

implementation

uses
  {$IFNDEF FMX}Controls,{$ENDIF}
  CRFunctions,
  DAConsts;

function DefConnectDialogClass: TClass;
begin
  Result := {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF};
end;

const
  KeyPath = '\SOFTWARE\Devart\LiteDAC\';

{ TLiteConnectDialog }

constructor {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.Create(Owner: TComponent);
begin
  inherited;

  Password.Order := 1;
  FDatabaseOption := TConnectDialogOption.Create(Self, okDatabase, 0);
  FDirectOption := TConnectDialogOption.Create(Self, okDirect, 2, False);
  FClientLibraryOption := TConnectDialogOption.Create(Self, okClientLibrary, 3, False);
  LabelSet := lsEnglish;
end;

destructor {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.Destroy;
begin
  FDatabaseOption.Free;
  FDirectOption.Free;
  FClientLibraryOption.Free;

  inherited;
end;

procedure {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.ReadDatabaseCaptionProperty(Reader: TReader);
begin
  FDatabaseOption.Caption := Reader.ReadString;
end;

procedure {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.ReadShowDatabaseProperty(Reader: TReader);
begin
  FDatabaseOption.Visible := Reader.ReadBoolean;
end;

procedure {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.ReadShowPasswordProperty(Reader: TReader);
begin
  Password.Visible := Reader.ReadBoolean;
end;

procedure {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.DefineProperties(Filer: TFiler);
begin
  inherited;
  Filer.DefineProperty('DatabaseLabel', ReadDatabaseCaptionProperty, nil, False);
  Filer.DefineProperty('ShowDatabase', ReadShowDatabaseProperty, nil, False);
  Filer.DefineProperty('ShowPassword', ReadShowPasswordProperty, nil, False);
end;

function {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.Execute: boolean;
var
  OldDatabase, OldEncryptionKey, OldClientLibrary: string;
  OldDirect: Boolean;
begin
  if Connection = nil then
    DatabaseError(SConnectionNotDefined);
  Assert(Connection is TLiteConnection);

  Result := False;
  OldDatabase := TLiteConnection(Connection).Database;
  OldEncryptionKey := TLiteConnection(Connection).EncryptionKey;
  OldDirect := TLiteConnection(Connection).Options.Direct;
  OldClientLibrary := TLiteConnection(Connection).ClientLibrary;
  try
    Result := inherited Execute;
  finally
    if not Result then begin
      TLiteConnection(Connection).Database := OldDatabase;
      TLiteConnection(Connection).EncryptionKey := OldEncryptionKey;
      TLiteConnection(Connection).Options.Direct := OldDirect;
      TLiteConnection(Connection).ClientLibrary := OldClientLibrary;
    end;
  end;
end;

function {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.DefDialogClass: TClass;
begin
  Result := TLiteConnectForm;
end;

function {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.GetConnection: TLiteConnection;
begin
  Result := (inherited Connection) as TLiteConnection;
end;

{$IFDEF MSWINDOWS}
function {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.GetServerStoreName: string;
begin
  Result := 'Database';
end;

function {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.GetKeyPath: string;
begin
  Result := KeyPath;
end;

procedure {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.SaveInfoToRegistry(Registry: TRegistry);
begin
  inherited;

  if Direct.Visible then
    Registry.WriteBool('Direct', TLiteConnection(Connection).Options.Direct)
  else
    Registry.DeleteValue('Direct');

  if Database.Visible then
    Registry.WriteString('Database', TLiteConnection(Connection).Database)
  else
    Registry.DeleteValue('Database');

  if ClientLibrary.Visible then
    Registry.WriteString('ClientLibrary', TLiteConnection(Connection).ClientLibrary)
  else
    Registry.DeleteValue('ClientLibrary');
end;

procedure {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.SaveServerListToRegistry;
var
  Registry: TRegistry;
  List: TStrings;
  i,j: integer;
  ServerKey: string;
begin
  if TLiteConnection(Connection).Database = '' then
    exit;

  Registry := TRegistry.Create(KEY_READ OR KEY_WRITE);
  try
    List := TStringList.Create;
    try
      LoadServerListFromRegistry(List); // call before creating Devart key

      if Registry.OpenKey(GetApplicationKeyPath, True) then begin
        ServerKey := GetServerStoreName;
        Registry.WriteString(Format('%s %d', [ServerKey, 1]), Connection.Database);
        i := 2;
        for j := 0 to List.Count - 1 do
          if not SameText(List[j], Connection.Database) then begin
            Registry.WriteString(Format('%s %d', [ServerKey, i]), List[j]);
            Inc(i);
          end;
        Registry.CloseKey;
      end;
    finally
      List.Free;
    end;
  finally
    Registry.Free;
  end;
end;

procedure {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.LoadInfoFromRegistry(Registry: TRegistry);
begin
  inherited;

  if Direct.Visible and Registry.ValueExists('Direct') then
    TLiteConnection(Connection).Options.Direct := Registry.ReadBool('Direct');

  if Database.Visible and Registry.ValueExists('Database') then
    TLiteConnection(Connection).Database := Registry.ReadString('Database');

  if ClientLibrary.Visible and Registry.ValueExists('ClientLibrary') then
    TLiteConnection(Connection).ClientLibrary := Registry.ReadString('ClientLibrary');
end;

procedure {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.LoadServerListFromRegistry(List: TStrings);
var
  ListKey, ServerKey: string;
  Registry: TRegistry;
  i: integer;
  KeyOpened: boolean;
begin
  Registry := TRegistry.Create(KEY_READ OR KEY_WRITE);
  try
    ListKey := GetApplicationKeyPath;
    KeyOpened := Registry.OpenKey(ListKey, False);
    if not KeyOpened then begin
      ListKey := StringReplace(ListKey, 'Devart', 'CoreLab', [rfIgnoreCase]);
      KeyOpened := Registry.OpenKey(ListKey, False);
    end;
    if KeyOpened then begin
      List.Clear;
      ServerKey := GetServerStoreName;
      i := 1;
      while Registry.ValueExists(Format('%s %d', [ServerKey, i])) do begin
        List.Add(Registry.ReadString(Format('%s %d', [ServerKey, i])));
        Inc(i);
      end;
      Registry.CloseKey;
    end;
  finally
    Registry.Free;
  end;
end;
{$ENDIF}

function {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.BrowseForDatabase: string;
var
  Dialog: TOpenDialog;
begin
  Dialog := TOpenDialog.Create(nil);
  try
  {$IFDEF MSWINDOWS}
    Dialog.Filter := 'SQLite Database Files (*.db3)|*.db3|All Files (*.*)|*.*';
  {$ELSE}
    Dialog.Filter := 'All Files (*)|*';
  {$ENDIF}
    Dialog.Options := Dialog.Options + [{$IFDEF VER16P}TOpenOption.{$ENDIF}ofFileMustExist];
    if Dialog.Execute then
      Result := Dialog.FileName;
  finally
    Dialog.Free;
  end;
end;

function {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.BrowseForClientLibrary: string;
var
  Dialog: TOpenDialog;
begin
  Dialog := TOpenDialog.Create(nil);
  try
  {$IFDEF MSWINDOWS}
    Dialog.Filter := 'DLL Files (*.dll)|*.dll|All Files (*.*)|*.*';
  {$ELSE}
    Dialog.Filter := 'All Files (*)|*';
  {$ENDIF}
    Dialog.Options := Dialog.Options + [{$IFDEF VER16P}TOpenOption.{$ENDIF}ofFileMustExist];
    if Dialog.Execute then
      Result := Dialog.FileName;
  finally
    Dialog.Free;
  end;
end;

procedure {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.SetLabelSet(Value: TLabelSet);
begin
  inherited SetLabelSet(Value);

  InSetLabelSet := True;
  try
  {$IFDEF WIN32_64}
    if Value <> lsCustom then begin
      FDatabaseOption.Caption := GetString(6);
    end;
  {$ELSE}
      FDatabaseOption.Caption := 'Database';
  {$ENDIF}
    Password.Caption := 'Password (encryption key)';
    FDirectOption.Caption := 'Mode';
    FClientLibraryOption.Caption := 'Client Library';
  finally
    InSetLabelSet := False;
  end;
end;

procedure {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.GetOptions(var Options: TConnectDialogOptionArray; Ordered: Boolean = True);
begin
  SetLength(Options, 4);
  Options[0] := Password;
  Options[1] := Database;
  Options[2] := Direct;
  Options[3] := ClientLibrary;
  inherited GetOptions(Options, Ordered);
end;

procedure {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.SetDatabaseOption(Value: TConnectDialogOption);
begin
  FDatabaseOption.Assign(Value);
end;

procedure {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.SetClientLibraryOption(Value: TConnectDialogOption);
begin
  FClientLibraryOption.Assign(Value);
end;

procedure {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.SetDirectOption(Value: TConnectDialogOption);
begin
  FDirectOption.Assign(Value);
end;

function {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.GetDatabaseLabel: string;
begin
  Result := FDatabaseOption.Caption;
end;

procedure {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.SetDatabaseLabel(Value: string);
begin
  FDatabaseOption.Caption := Value;
end;

function {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.GetShowDatabase: Boolean;
begin
  Result := FDatabaseOption.Visible;
end;

procedure {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.SetShowDatabase(Value: Boolean);
begin
  FDatabaseOption.Visible := Value;
end;

function {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.GetShowPassword: Boolean;
begin
  Result := Password.Visible;
end;

procedure {$IFDEF FMX}TLiteConnectDialogFmx{$ELSE}TLiteConnectDialog{$ENDIF}.SetShowPassword(Value: Boolean);
begin
  Password.Visible := Value;
end;

initialization
{$IFDEF FMX}
  if not Assigned(DefConnectDialogClassProc) then
{$ENDIF}
    DefConnectDialogClassProc := DefConnectDialogClass;

{$IFDEF VER16P}
{$IFDEF FMX}
  GroupDescendentsWith(TLiteConnectDialogFmx, TFmxObject);
{$ELSE}
  GroupDescendentsWith(TLiteConnectDialog, TControl);
{$ENDIF}
{$ENDIF}

end.
